<?php
/*
 +-------------------------------------------------------------------------+
 | Copyright (C) 2006 Reinhard Scheck aka gandalf                          |
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
*/

#global $config;
/* let's report all errors */
#error_reporting(E_ALL);

/**
 * Initialize the plugin and setup all hooks
 */
function plugin_init_aggregate() {
	global $config, $plugin_hooks;

	# Add a new dropdown Action for Graph Management
	$plugin_hooks['graphs_action_array']['aggregate'] = 'aggregate_graphs_action_array';
	
	# setup all arrays needed for aggregation
#	$plugin_hooks['config_arrays']['aggregate'] = 'aggregate_config_arrays';
	
	# setup all forms needed for aggregation
#	$plugin_hooks['config_form']['aggregate'] = 'aggregate_config_form';
    
	# provide navigation texts
#	$plugin_hooks['draw_navigation_text'] ['aggregate'] = 'aggregate_draw_navigation_text';
	
	# Graph Management Action dropdown selected: prepare the list of graphs for a confirmation request
	$plugin_hooks['graphs_action_prepare']['aggregate'] = 'aggregate_graphs_action_prepare';
	
	# Graph Management Action dropdown selected: execute list of graphs
	$plugin_hooks['graphs_action_execute']['aggregate'] = 'aggregate_graphs_action_execute';
	
}

/**
 * Version information (used by update plugin)
 */
function aggregate_version () {
#	if (read_config_option("log_verbosity", TRUE) == POLLER_VERBOSITY_DEBUG) {
#		cacti_log("Function: aggregate_version called", FALSE, "AGGREGATE");
#	}
	
	return array( 'name' 	=> 'aggregate',
			'version' 	=> '0.2',
			'longname'	=> 'Create Aggregate Graphs',
			'author'	=> 'Reinhard Scheck',
			'homepage'	=> 'http://cactiusers.org',
			'email'		=> '',
#			'url'		=> 'http://cactiusers.org/cacti/versions.php'
			'url'		=> ''
			);
}

/**
 * Draw navigation texts
 * @arg $nav		all navigation texts
 */
function aggregate_draw_navigation_text ($nav) {
    // Displayed navigation text under the blue tabs of Cacti
    $nav["aggregate_templates.php:"] 				= array("title" => "Aggregate Templates", "mapping" => "index.php:", "url" => "aggregate_templates.php", "level" => "1");
	$nav["aggregate_templates.php:template_edit"] 	= array("title" => "(Edit)", "mapping" => "index.php:,aggregate_templates.php:", "url" => "", "level" => "2");
	$nav["aggregate_templates.php:actions"] 		= array("title" => "Actions", "mapping" => "index.php:,aggregate_templates.php:", "url" => "", "level" => "2");
	$nav["aggregate_templates_items.php:item_edit"] = array("title" => "Aggregate Template Items", "mapping" => "index.php:,aggregate_templates.php:,aggregate_templates.php:template_edit", "url" => "", "level" => "3");

	$nav["color_templates.php:"] 				= array("title" => "Color Templates", "mapping" => "index.php:", "url" => "color_templates.php", "level" => "1");
	$nav["color_templates.php:template_edit"] 	= array("title" => "(Edit)", "mapping" => "index.php:,color_templates.php:", "url" => "", "level" => "2");
	$nav["color_templates.php:actions"] 		= array("title" => "Actions", "mapping" => "index.php:,color_templates.php:", "url" => "", "level" => "2");
	$nav["color_templates_items.php:item_edit"] = array("title" => "Color Template Items", "mapping" => "index.php:,color_templates.php:,color_templates.php:template_edit", "url" => "", "level" => "3");
	
    return $nav;
}

/**
 * Setup the new dropdown action for Graph Management
 * @arg $action		actions to be performed from dropdown
 */
function aggregate_graphs_action_array($action) {
#	if (read_config_option("log_verbosity", TRUE) == POLLER_VERBOSITY_DEBUG) {
#		cacti_log("Function: aggregate_graphs_action_array called", FALSE, "AGGREGATE");
#	}
	$action['plugin_aggregate'] = 'Create Aggregate Graph';
	return $action;
}

/**
 * PHP error handler
 * @arg $errno
 * @arg $errmsg
 * @arg $filename
 * @arg $linenum
 * @arg $vars
 */
function aggregate_error_handler($errno, $errmsg, $filename, $linenum, $vars) {
	if (read_config_option("log_verbosity") >= POLLER_VERBOSITY_HIGH) {
		/* define all error types */
		$errortype = array(
			E_ERROR             => 'Error',
			E_WARNING           => 'Warning',
			E_PARSE             => 'Parsing Error',
			E_NOTICE            => 'Notice',
			E_CORE_ERROR        => 'Core Error',
			E_CORE_WARNING      => 'Core Warning',
			E_COMPILE_ERROR     => 'Compile Error',
			E_COMPILE_WARNING   => 'Compile Warning',
			E_USER_ERROR        => 'User Error',
			E_USER_WARNING      => 'User Warning',
			E_USER_NOTICE       => 'User Notice',
#			E_STRICT            => 'Runtime Notice',
#			E_RECOVERABLE_ERROR => 'Catchable Fatal Error'
		);

		/* create an error string for the log */
		$err = "ERRNO:'"  . $errno   . "' TYPE:'"    . $errortype[$errno] .
			"' MESSAGE:'" . $errmsg  . "' IN FILE:'" . $filename .
			"' LINE NO:'" . $linenum . "'";

		/* let's ignore some lesser issues */
		if (substr_count($errmsg, "date_default_timezone")) return;
		if (substr_count($errmsg, "Only variables")) return;

		/* log the error to the Cacti log */
#		cacti_log("PROGERR: " . $err, false, "aggregate");
		print("PROGERR: " . $err . "<br><pre>");# print_r($vars); print("</pre>");
	}

	return;
}

/**
 * create a new graph based on an existing one
 * take all basic graph definitions from this one, but omit graph items
 * wipe out host_id and graph_template_id
 * @arg $_local_graph_id		id of an already existing graph
 * return				id of the new graph
 *  */
function aggregate_graphs_create_graph($_local_graph_id) {
	global $struct_graph, $struct_graph_item;

	/* suppress warnings */
	error_reporting(0);

	/* install the boost error handler */
	set_error_handler("aggregate_error_handler");
	
#	if (read_config_option("log_verbosity", TRUE) == POLLER_VERBOSITY_DEBUG) {
#		cacti_log("function: aggregate_graphs_create_graph from: " . $_local_graph_id, true, "AGGREGATE");
#	}

	# suppress templating
#	$_graph_template_id = "";

	if (!empty($_local_graph_id)) {
		$graph_local = db_fetch_row("select * from graph_local where id=$_local_graph_id");
		$graph_template_graph = db_fetch_row("select * from graph_templates_graph where local_graph_id=$_local_graph_id");
		#$graph_template_items = db_fetch_assoc("select * from graph_templates_item where local_graph_id=$_local_graph_id");

		/* create new entry: graph_local */
		$save["id"] = 0;
		$save["graph_template_id"] = 0; 	# no templating
		$save["host_id"] = 0;  				# no host to be referred to
		$save["snmp_query_id"] = 0;			# no templating
		$save["snmp_index"] = 0;			# no templating
		# insert the new graph
		$local_graph_id = sql_save($save, "graph_local");

		# apply given graph title
		$graph_title = $_POST["title_format"]; 
		$graph_template_graph["title"] = str_replace("<graph_title>", $graph_template_graph["title"], $graph_title);
		$graph_template_id = sql_save($save, "graph_templates");
	}

	unset($save);
	reset($struct_graph);

	/* create new entry: graph_templates_graph */
	$save["id"] = 0;
	$save["local_graph_id"] = (isset($local_graph_id) ? $local_graph_id : 0);
	$save["local_graph_template_graph_id"] = 0; 	# no templating
	$save["graph_template_id"] = 0; 				# no templating
	$save["title_cache"] = $graph_template_graph["title_cache"];

	# copy all t_ fields
	reset($struct_graph);
	while (list($field, $array) = each($struct_graph)) {
		$save{$field} = $graph_template_graph{$field};
		$save{"t_" . $field} = $graph_template_graph{"t_" . $field};
	}
	# insert the new graph
	$graph_templates_graph_id = sql_save($save, "graph_templates_graph");

	# update title cache
	if (!empty($_local_graph_id)) {
		update_graph_title_cache($local_graph_id);
	}

	/* restore original error handler */
	restore_error_handler();
	
	# return the id of the newly inserted graph
	return $local_graph_id;
	
}

/**
 * inserts all graph items of an existing graph
 * @arg $_new_graph_id				id of the new graph
 * @arg $_old_graph_id				id of the old graph
 * @arg $_aggregate_template_items	aggregate_template_items array
 * @arg $_sequence					sequence number of the next graph item to be inserted
 * return						id of the next graph item to be inserted
 *  */
function aggregate_graphs_insert_graph_items($_new_graph_id, $_old_graph_id, $_sequence) {
	global $struct_graph_item;

	/* suppress warnings */
	error_reporting(0);

	/* install the boost error handler */
	set_error_handler("aggregate_error_handler");
	
#	if (read_config_option("log_verbosity", TRUE) == POLLER_VERBOSITY_LOW) {
#		cacti_log("function: aggregate_graphs_insert_graph_items called. Insert " . $_old_graph_id . " into " . $_new_graph_id . " sequence: " . $_sequence, true, "AGGREGATE");
#	}
	
#	$graph_template_id = "";

	# take basic graph data from new graph entry
#	if (!empty($_new_graph_id)) {
#		$graph_local = db_fetch_row("select * from graph_local where id=$_new_graph_id");
#		$graph_template_graph = db_fetch_row("select * from graph_templates_graph where local_graph_id=$_new_graph_id");
#	}
	
	# take graph item data from old one
	if (!empty($_old_graph_id)) {
		$graph_template_items = db_fetch_assoc("select * from graph_templates_item where local_graph_id=$_old_graph_id ORDER BY SEQUENCE");
	}
	
	/* create new entry(s): graph_templates_item */
	$num_items = sizeof($graph_template_items);
	if ($num_items > 0) {
		$i = 1;
		# next entry will have to have a prepended text format
		$prepend = true; 
		foreach ($graph_template_items as $graph_template_item) {
#			if (read_config_option("log_verbosity", TRUE) == POLLER_VERBOSITY_LOW) {
#				cacti_log("function: sequence: " . $_sequence . "/" . $graph_template_item["sequence"] . " loop: " . $i, true, "AGGREGATE");
#			}
			unset($save);
			reset($struct_graph_item);

			$save["id"] = 0;
			$save["local_graph_id"] = (isset($_new_graph_id) ? $_new_graph_id : 0);
			$save["graph_template_id"] = (!empty($_new_graph_id) ? $graph_template_item["graph_template_id"] : $graph_template_id);
			$save["local_graph_template_item_id"] = (isset($graph_template_item["local_graph_template_item_id"]) ? $graph_template_item["local_graph_template_item_id"] : 0);
			
			while (list($field, $array) = each($struct_graph_item)) {
				$save{$field} = $graph_template_item{$field};
#				if (read_config_option("log_verbosity", TRUE) == POLLER_VERBOSITY_LOW) {
#					cacti_log("function: aggregate_graphs_insert_graph_items field:" . $field . " value: " . $save{$field}, true, "AGGREGATE");
#				}
			}
			# prepend text format for this item? Only, if text was given ...
			if ($prepend && (strlen($save["text_format"]) > 0)) {
				# legend items will look nicer, when prepended with the hostname
				# get hostname for current graph item
				$sql = "SELECT host.hostname " . 
						"FROM host, graph_local, graph_templates_item " .
						"WHERE graph_templates_item.local_graph_id = graph_local.id AND " .
						"graph_local.host_id = host.id AND " .
						"graph_templates_item.id = " . $graph_template_item["id"];
				$hostname = db_fetch_cell($sql);
				$save["text_format"] = $hostname . " " . $save["text_format"];
				# no more prepending until next line break is encountered
				$prepend = false; 
			}

			# last item?
			if ($num_items == $i) {
				# set line break on, some templates omit line break on last item
				# looks ugly if not dealt with
				$save["hard_return"] = "on";
			}
			
			# provide new sequence number
			$save["sequence"] = $_sequence;
#			if (read_config_option("log_verbosity", TRUE) == POLLER_VERBOSITY_LOW) {
#				cacti_log("function: aggregate_graphs_insert_graph_items hard return: " . $save["hard_return"] . " sequence: " . $_sequence, true, "AGGREGATE");
#			}
			
			# if this item defines a line break, remember to prepend next line
			$prepend = ($save["hard_return"] == "on");
		
			$graph_item_mappings{$graph_template_item["id"]} = sql_save($save, "graph_templates_item");

			$i++;
			$_sequence++;
		}
	}

	/* restore original error handler */
	restore_error_handler();
	
	# return with next sequence number to be filled
	return $_sequence;
}

/**
 * cleanup of graph items of the new graph
 * @arg $_graph_id			id of the new graph
 * @arg $_color_spacing		offset for colored items
 * return				-
 *  */
function aggregate_graphs_cleanup($_graph_id, $_color_spacing) {
	global $struct_graph, $struct_graph_item;

	/* suppress warnings */
	error_reporting(0);

	/* install the boost error handler */
	set_error_handler("aggregate_error_handler");
	
#	if (read_config_option("log_verbosity", TRUE) == POLLER_VERBOSITY_DEBUG) {
#		cacti_log("function: aggregate_graphs_cleanup called. Working on Graph: " . $_graph_id . " color spacing: " . $_color_spacing, true, "AGGREGATE");
#	}
	
	# fetch all items
	if (!empty($_graph_id)) {
		$graph_template_items = db_fetch_assoc("select * from graph_templates_item where local_graph_id=$_graph_id ORDER BY sequence");
	}

	/* loop through all graph_templates_item */
	# sequence number counting only colored items
	$i = 0;
	if (sizeof($graph_template_items) > 0) {
		foreach ($graph_template_items as $graph_template_item) {
			unset($save);
			reset($struct_graph_item);

			if ($graph_template_item["color_id"] > 0) {
				$base = $graph_template_item["color_id"];
				$offset = $_color_spacing*$i++;
				$new_color = $base + $offset;

				$sql = "UPDATE graph_templates_item SET color_id =" . 
						$new_color . 
						" WHERE id=" . 
						$graph_template_item["id"]
						;
				# perform updating of color_id field
				db_execute($sql);
			}
		}
	}

	/* restore original error handler */
	restore_error_handler();
	
}

/**
 * perform aggregate_graph execute action
 * @arg $action				action to be performed
 * return				-
 *  */
function aggregate_graphs_action_execute($action) {
#	global $colors, $graph_actions;

	/* suppress warnings */
	error_reporting(0);

	/* install the boost error handler */
	set_error_handler("aggregate_error_handler");
	
#	if (read_config_option("log_verbosity", TRUE) == POLLER_VERBOSITY_LOW) {
#		cacti_log("function: aggregate_graphs_action_execute called. Action: " . $action, true, "AGGREGATE");
# 	}

	if (isset($_POST["selected_items"])) {
		$selected_items = unserialize(stripslashes($_POST["selected_items"]));
		
		if ($action == "plugin_aggregate") { /* aggregate */
			# create new graph based on first graph selected
			$new_graph_id = aggregate_graphs_create_graph($selected_items[0]);
			# sequence number of next graph item to be added
			$next_item_sequence = 0;
	
			# loop for all selected graphs
			for ($i=0;($i<count($selected_items));$i++) {
				/* ================= input validation ================= */
				input_validate_input_number($selected_items[$i]);
				/* ==================================================== */
				# insert all graph items of selected graph
				# next items to be inserted have to be in sequence
				$next_item_sequence = aggregate_graphs_insert_graph_items($new_graph_id, $selected_items[$i], $next_item_sequence);
			}
			#aggregate_graphs_cleanup($new_graph_id, $_POST["color_spacing"]);
		}
	}

	/* restore original error handler */
	restore_error_handler();
}

/**
 * 
 * perform aggregate_graph 				prepare action
 * @arg $save			Array: 
 *						drp_action:		selected action from dropdown
 *						graph_array:	graphs titles selected from graph management's list
 *						graph_list:		graphs selected from graph management's list
 * return				-
 *  */
function aggregate_graphs_action_prepare($save) {
	# globals used
	global $colors;
	
	/* suppress warnings */
	error_reporting(0);

	/* install the boost error handler */
	set_error_handler("aggregate_error_handler");
	
#	if (read_config_option("log_verbosity", TRUE) == POLLER_VERBOSITY_DEBUG) {
#		cacti_log("function: aggregate_graphs_action_prepare called.", true, "AGGREGATE");
#	}

	# it's our turn
	if ($save["drp_action"] == "plugin_aggregate") { /* aggregate */
		$graphs = array();
		
		/* find out which (if any) data sources are being used by this graph, so we can tell the user */
		if (isset($save["graph_array"])) {
	
			# provide a new prefix for graph titles
			# shall we make it configurable via "Settings"?
			$graph_prefix = "Aggregate"; 
	
			# get all aggregate templates
			$aggregate_templates = db_fetch_assoc("select
				aggregate_template_id as id,
				name as name
				from plugin_aggregate_templates
				order by name ASC");
			//print "<pre>";print_r($aggregate_templates);print "</pre>";
			
			$data_sources = db_fetch_assoc("select
				data_template_data.local_data_id,
				data_template_data.name_cache
				from (data_template_rrd,data_template_data,graph_templates_item)
				where graph_templates_item.task_item_id=data_template_rrd.id
				and data_template_rrd.local_data_id=data_template_data.local_data_id
				and " . array_to_sql_or($save["graph_array"], "graph_templates_item.local_graph_id") . "
				and data_template_data.local_data_id > 0
				group by data_template_data.local_data_id
				order by data_template_data.name_cache");
		}	

		print "	<tr>
				<td class='textArea' bgcolor='#" . $colors["form_alternate1"]. "'>
				<p>Are you sure you want to aggregate the following graphs?</p>
				<p>" . $save["graph_list"] . "</p>
				<p><strong>Title Format:</strong><br>"; form_text_box("title_format", $graph_prefix . " <graph_title>", "", "255", "30", "text"); print "</p>
				";

		if (sizeof($data_sources) > 0) {
			print "<tr bgcolor='#" . $colors["form_alternate1"] . "'><td class='textArea'><p class='textArea'>The following data sources are in use by these graphs:</p>\n";
			foreach ($data_sources as $data_source) {
				print "<strong>" . $data_source["name_cache"] . "</strong><br>\n";
			}	
		}

		print "
			</td>
			</tr>\n
			";
	}

	/* restore original error handler */
	restore_error_handler();
}

/**
 * Setup arrays needed for this plugin
 */
function aggregate_config_arrays () {
	# globals used
	# ... none
	# globals changed
	global $user_auth_realms, $user_auth_realm_filenames, $menu;
	# globals defined
	global $aggregate_item_function, $aggregate_item_types;
	
	# realms
	# Check this Item for each user to allow access to Aggregate Templates
	$user_auth_realms[72]='Plugin -> Create Aggregate Templates';
	# these are the files protected by our realm id
	$user_auth_realm_filenames['aggregate_templates.php'] = 72;
	$user_auth_realm_filenames['aggregate_templates_items.php'] = 72;
	$user_auth_realm_filenames['color_templates.php'] = 72;
	$user_auth_realm_filenames['color_templates_items.php'] = 72;
	
	# menu titles
	$menu["Templates"]['plugins/aggregate/aggregate_templates.php'] = "Aggregate Templates";
	$menu["Templates"]['plugins/aggregate/color_templates.php'] = "Color Templates";
	
	# specific arrays made global for use within this plugin
	$aggregate_item_types = array(1 =>
		"LINE1",
		"LINE2",
		"LINE3",
		"AREA",
		"STACK",
		);

	$aggregate_item_function = array(1 =>
		"Keep Item",
		"Skip Item",
		);
		
}

/**
 * Setup forms needed for this plugin
 */
function aggregate_config_form () {
	# lobals used
	global $graph_item_types, $consolidation_functions;
	# globals defined for use with Aggregate Templates
	global $struct_aggregate, $struct_aggregate_item, $fields_aggregate_template_template_edit, $aggregate_item_function;
	# globals defined for use with Color Templates
	global $struct_color_template, $struct_color_template_item, $fields_color_template_template_edit;
	
	# ------------------------------------------------------------
	# Aggregate Templates
	# ------------------------------------------------------------
	/* file: aggregate_templates.php, action: template_edit */
	$struct_aggregate = array(
		"title" => array(
			"friendly_name" => "Title",
			"method" => "textbox",
			"max_length" => "255",
			"default" => "",
			"description" => "The name of this Aggregate Template."
			),
		);

	/* file: aggregate_templates.php, action: item_edit */
	$struct_aggregate_item = array(
		"aggregate_function" => array(
			"friendly_name" => "Function",
			"method" => "drop_array",
			"array" => $aggregate_item_function,
			"default" => "0",
			"description" => "Keep or skip this item."
			),
		"color_id" => array(
			"friendly_name" => "Color",
			"method" => "drop_color",
			"default" => "0",
			"description" => "The color to use for the legend.<br>
				Use <strong>NONE</strong> to keep current color settings."
			),
		"graph_type_id" => array(
			"friendly_name" => "Graph Item Type",
			"method" => "drop_array",
			"array" => $graph_item_types,
			"default" => "0",
			"none_value" => "Keep Item Type",
			"description" => "How data for this item is represented visually on the graph.<br>
				Use any value to <strong>override</strong> current settings."
			),
		"consolidation_function_id" => array(
			"friendly_name" => "Consolidation Function",
			"method" => "drop_array",
			"array" => $consolidation_functions,
			"default" => "0",
			"none_value" => "Keep Consolidation Function",
			"description" => "How data for this item is represented statistically on the graph.<br>
				Use any value to <strong>override</strong> current settings."
			),
		"cdef_id" => array(
			"friendly_name" => "CDEF Function",
			"method" => "drop_sql",
			"sql" => "select id,name from cdef order by name",
			"default" => "0",
			"none_value" => "Keep CDEF",
			"description" => "A CDEF (math) function to apply to this item on the graph.<br>
				Use any value to <strong>override</strong> current settings."
			),
		"value" => array(
			"friendly_name" => "Value",
			"method" => "textbox",
			"max_length" => "50",
			"default" => "",
			"description" => "The value of an HRULE or VRULE graph item."
			),
		"gprint_id" => array(
			"friendly_name" => "GPRINT Type",
			"method" => "drop_sql",
			"sql" => "select id,name from graph_templates_gprint order by name",
			"default" => "2",
			"description" => "If this graph item is a GPRINT, you can optionally choose another format
				here. You can define additional types under \"GPRINT Presets\"."
			),
		"text_format" => array(
			"friendly_name" => "Text Format",
			"method" => "textbox",
			"max_length" => "255",
			"default" => "",
			"description" => "Text that will be displayed on the legend for this graph item."
			),
		"aggregate_sequence" => array(
			"friendly_name" => "Aggregate Sequence",
			"method" => "textbox",
			"max_length" => "255",
			"default" => "",
			"description" => "Sequence Number for the Aggregated Item.<br>
				Defaults to Sequence take from original Graph Item.<br>
				Override e.g. for AREA/STACK on negative y-axis"
			),
		"hard_return" => array(
			"friendly_name" => "Insert Hard Return",
			"method" => "checkbox",
			"default" => "",
			"description" => "Forces the legend to the next line after this item.<br>
				Leave unchecked to keep current setting.<br>
				Check to <strong>override</strong> current settings."
			),
		"sequence" => array(
			"friendly_name" => "Sequence",
			"method" => "view"
			)
		);

	/* file: aggregate_templates.php, action: template_edit */
	$fields_aggregate_template_template_edit = array(
		"name" => array(
			"method" => "textbox",
			"friendly_name" => "Name",
			"description" => "A useful name for this Template.",
			"value" => "|arg1:name|",
			"max_length" => "255",
			),
		"aggregate_template_id" => array(
			"method" => "hidden_zero",
			"value" => "|arg1:aggregate_template_id|"
			),
		"save_component_aggregate" => array(
			"method" => "hidden",
			"value" => "1"
			)
		);
	
	# ------------------------------------------------------------
	# Color Templates
	# ------------------------------------------------------------
	/* file: color_templates.php, action: template_edit */
	$struct_color_template = array(
		"title" => array(
			"friendly_name" => "Title",
			"method" => "textbox",
			"max_length" => "255",
			"default" => "",
			"description" => "The name of this Color Template."
			),
		);

	/* file: color_templates.php, action: item_edit */
	$struct_color_template_item = array(
		"color_id" => array(
			"friendly_name" => "Color",
			"method" => "drop_color",
			"default" => "0",
			"description" => "A nice Color"
			),
		);

	/* file: color_templates.php, action: template_edit */
	$fields_color_template_template_edit = array(
		"name" => array(
			"method" => "textbox",
			"friendly_name" => "Name",
			"description" => "A useful name for this Template.",
			"value" => "|arg1:name|",
			"max_length" => "255",
			),
		"color_template_id" => array(
			"method" => "hidden_zero",
			"value" => "|arg1:color_template_id|"
			),
		"save_component_color" => array(
			"method" => "hidden",
			"value" => "1"
			)
		);
		
}

?>